#ifndef __BIT_H
# define __BIT_H

/**
 * return a value where the __b-th bit is set
 */
#define BIT_SET(__b)            (1ull << (__b))

/**
 * Return number of bits needed to store the type
 * of an expression
 */
#define BITS_COUNT(__x)         (sizeof(__x)*8)

/**
 * Return the fsb of an expression
 * NOTE: since its a macro, this may double evaluate its argument !
 */
#define FSB(__x)                ((__x) & -(__x))

// Set, unset or flip the `__b`^th bit of `__v`
#define BS_SET(__v, __b)        ((__v) | BIT_SET(__b))
#define BS_FLIP(__v, __b)       ???
#define BS_UNSET(__v, __b)      ???
#define BS_UNSET_FSB(__v)       ???


/**
 * Retourne le nombre de bits à 1 dans un unsigned long long `n`
 *
 * Référez-vous à l'exercice bit_count pour des idées d'algorithmes
 */
int bit_count(unsigned long long n);

/**
 * Retourne l'indice du bit de poids le plus faible actif (fsb). Les indices
 * commencent à 1. Cette fonction retourne zéro si aucun bit n'est activé
 * c'est à dire ssi i == 0
 *
 * Cette fonction existe sous le nom `ffs` (ffsll).
 */
int fsb_pos(unsigned long long i);

enum endian { LITTLE, BIG };

/**
 * Convertit `size` octets du tableau `data`, en leur représentation native
 * (en utilisant la méthode de la multiplication [ou décalage à gauche]).
 * Les valeurs de `data` sont stockés en `endianness` endian.
 *
 * Exemples:
 *      4, {0xEF, 0xBE, 0xAD, 0xDE}, LITTLE  ==> 0xDEADBEEF
 *      2, {0xCA, 0xFE}, BIG                 ==> 0xCAFE
 */
unsigned long long convert_endian2n(
        int size, const unsigned char data[], enum endian endianness);

#endif
