#ifndef __BUZZER_H
# define __BUZZER_H

#include <stdio.h>

#define INPUT_DEVICE    "/dev/hidraw0"
#define MAX_PLAYERS     4

// Length to read/write in bytes
#define WRITE_LEN       8
#define READ_LEN        8

#define BUTTON_COUNT    5
#define PLAYER_OFFSET   16
#define PLAYER_MASK     ((1 << BUTTON_COUNT) - 1)
#define PLAYER(__p, __w) ???

// sorted from lsb to msb.
enum Buttons { RED = 0, YELLOW, GREEN, ORANGE, BLUE };

/**
 * Open a buzzer device identified by `fname`.
 * If `fname` is null INPUT_DEVICE will be used.
 * return a valid file descriptor if success, NULL otherwise
 */
FILE *open_buzzer(const char *fname);

/**
 * Read `status` from an openened buzzer (`fd`)
 * return 0 if success.
 */
int read_status(FILE *fd, unsigned long long *status);

/**
 * Send `status` to an openened buzzer (`fd`).
 * return 0 if success.
 */
int send_status(FILE* fd, unsigned long long *status);

/**
 * Return a bitset of players who have played.
 */
int has_played(unsigned long long *w);

/**
 * Fill buffer with lights specified in `set`.
 * Return the input buffer.
 */
void set_lights(int set, unsigned long long *buffer);

/**
 * (re)initilize buffer.
 * Return the input buffer
 */
void reset_buffer(unsigned long long *buffer);

/**
 * repeat `repeat` times an animation of `animcount` buzzer buffers (`animation`).
 * `interval` useconds (u = 10^-6) will be waited in between two frames.
 */
void animate(int repeat, unsigned int interval, int animcount, unsigned long long *animation[], FILE* fd);

// Button name
const char *button_name(enum Buttons b);

/**
 * Utility function which exit the program if `condition` is met. Use `msg` as
 * error message prefix.
 */
void exit_if(int condition, const char *msg);
#endif
