#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "bit.h"
#include "buzzer.h"

#define SENTINEL        (0xf0ull << 32)

#define B(__v)          (1ull<<(__v))

#define MK_PLAYER(__p, __v) \
    ((((__v) & PLAYER_MASK) << ((__p)*BUTTON_COUNT + PLAYER_OFFSET)) | SENTINEL)

const unsigned long long values[] =
{
    MK_PLAYER(0, B(RED)),
    MK_PLAYER(1, B(RED)),
    MK_PLAYER(2, B(RED)),
    MK_PLAYER(3, B(RED)),
    MK_PLAYER(1, B(YELLOW)|B(GREEN)) | MK_PLAYER(2, B(ORANGE)|B(BLUE)),
    MK_PLAYER(0, B(RED)) | MK_PLAYER(1, B(RED))
        | MK_PLAYER(2, B(RED)) | MK_PLAYER(3, B(RED)),
    MK_PLAYER(3, B(RED)|B(YELLOW)|B(GREEN)|B(ORANGE)|B(BLUE)),
};

void convert_n2endian(unsigned long long value,
        int size, unsigned char data[], enum endian endianness)
{
    int offset = (endianness == BIG) ? size - 1 : 0;
    int direction = (endianness == BIG) ? -1 : 1;
    while (size --) {
        data[offset] = value & 0xFF;
        value >>= 8;
        offset += direction;
    }
}

void exit_if(int condition, const char *msg)
{
    if (condition) {
        perror(msg);
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
    int delay = 0;
    if (argc > 2 && strcmp("--delay", argv[1]) == 0) {
        delay = atoi(argv[2]);
        argc -= 2;
        argv += 2;
    }

    FILE *fd = stdout;
    if (argc > 1 && strcmp("-", argv[1]) != 0) {
        fd = fopen(argv[1], "wb");
        exit_if(fd == NULL, "fopen");
    }
    exit_if(setvbuf(fd, NULL, _IONBF, 0) != 0, "setvbuf");

    for (int i = 0; i < sizeof(values)/sizeof(values[0]); ++i) {
        unsigned char buffer[READ_LEN];
        fprintf(stderr, "writing %16llx\n", values[i]);

        convert_n2endian(values[i], READ_LEN, buffer, LITTLE);
        exit_if(
                fwrite(buffer, READ_LEN, 1, fd) == 0, "fwrite");
        sleep(delay);
    }

    fclose(fd);
    return 0;
}

